<?php

namespace Drupal\alt_text_import_csv;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\StreamWrapper\PublicStream;
use Drupal\Core\Url;
use Drupal\file\FileInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\path_alias\AliasManagerInterface;

/**
 * Imports alt texts given page and image URLs.
 */
class AltTextImporter {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file repository service.
   *
   * @var \Drupal\file\FileRepositoryInterface
   */
  protected $fileRepository;

  /**
   * The alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Creates a AltTextImporter instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\file\FileRepositoryInterface $file_repository
   *   The file repository service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The alias manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileRepositoryInterface $file_repository,
    AliasManagerInterface $alias_manager
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileRepository = $file_repository;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Imports alt text.
   *
   * @param string $image_url
   *   The URL of the image to update.
   * @param string $page_url
   *   The URL of the page on which the image is seen.
   * @param string $alt_text
   *   The new alt text to set.
   *
   * @throws \InvalidArgumentException
   *   Throws an exception if one of the following applies:
   *   - An entity cannot be determined from the $page_url.
   *   - An image file entity cannot be determined from the $image_url.
   *   - The image file cannot be found in the fields on the entity.
   */
  public function importAltText(string $image_url, string $page_url, string $alt_text) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->getEntityFromUrl($page_url);

    $file = $this->getImageFromUrl($image_url);

    // Work through all reference fields on the entity which point to image
    // files or media. Keep track of updates to alt text so we can complain if
    // nothing was done.
    $alt_texts_updated = [];
    /** @var \Drupal\Core\Field\FieldDefinitionInterface $field */
    foreach ($entity->getFieldDefinitions(FALSE) as $field_name => $field) {
      $field_type = $field->getType();

      $referenced_type = match ($field_type) {
        'image' => 'file',
        // @todo Use EntityReferenceItemInterface on 10.2.
        'entity_reference' => $field->getSetting('target_type'),
        default => NULL,
      };

      if (!$referenced_type || !in_array($referenced_type, ['file', 'media'])) {
        continue;
      }

      // Try to find the file in a media or file field.
      if ($referenced_type == 'file') {
        $referenced_files = $entity->get($field_name)->referencedEntities();
        foreach ($referenced_files as $delta => $referenced_file) {
          if ($referenced_file->id() == $file->id()) {
            // Update the alt text on the host entity.
            $this->updateAltText($entity, $field, $delta, $alt_text);
            $alt_texts_updated[] = TRUE;
          }
        }
      }
      if ($referenced_type == 'media') {
        foreach ($entity->get($field_name)->referencedEntities() as $delta => $media_entity) {
          // @todo Consider other media source plugins which handle images!
          $media_source_plugin = $media_entity->getSource();
          if ($media_source_plugin->getPluginId() == 'image') {
            if ($image_file_id = $media_source_plugin->getSourceFieldValue($media_entity)) {
              if ($image_file_id == $file->id()) {
                $media_type = $this->entityTypeManager->getStorage('media_type')->load($media_entity->bundle());
                $source_field = $media_source_plugin->getSourceFieldDefinition($media_type);

                // Update the alt text on the media entity.
                $this->updateAltText($media_entity, $source_field, 0, $alt_text);
                $alt_texts_updated[] = TRUE;
              }
            }
          }
        }
      }
    } // foreach $field

    if (empty($alt_texts_updated)) {
      throw new \InvalidArgumentException("The file was not found in the entity at the given path.");
    }
  }

  /**
   * Updates the alt text for the image file field in an entity.
   *
   * The entity will be saved.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The host entity.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $file_field
   *   The field holding the image reference.
   * @param integer $delta
   *   The delta index of the field in which to update the alt text.
   * @param string $alt_text
   *   The new value for the alt text.
   */
  protected function updateAltText(ContentEntityInterface $entity, FieldDefinitionInterface $file_field, int $delta, string $alt_text) {
    $entity->get($file_field->getName())->get($delta)->get('alt')->setValue($alt_text);
    $entity->save();
  }

  /**
   * Gets the entity for a given URL.
   *
   * @param string $page_url
   *   The page URL. This can be a path alias or the real path.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity.
   *
   * @throws \InvalidArgumentException
   *   Throws an exception if an entity can't be found.
   */
  protected function getEntityFromUrl(string $page_url): EntityInterface {
    // Get the entity path from the URL.
    $path = parse_url($page_url, PHP_URL_PATH);
    // Remove the base path, if Drupal is running in a subfolder. We subtract 1
    // from the string length because the base path includes the terminal '/'
    // but we need it at the start of the path.
    $path = substr($path, strlen(base_path()) - 1);

    $path = $this->aliasManager->getPathByAlias($path);

    // Figure out the entity this path is for.
    $params = Url::fromUri("internal:" . $path)->getRouteParameters();
    $entity_type = key($params);
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($params[$entity_type]);

    if (!$entity) {
      throw new \InvalidArgumentException("No entity found for the path $page_url.");
    }

    return $entity;
  }

  /**
   * Gets a file entity from the image URL.
   *
   * @param string $image_url
   *   The image URL.
   *
   * @return \Drupal\file\FileInterface
   *   The file entity.
   *
   * @throws \InvalidArgumentException
   *   Throws an InvalidArgumentException if no file entity can be found.
   */
  protected function getImageFromUrl(string $image_url): FileInterface {
    // Trim off the domain.
    $path = parse_url($image_url, PHP_URL_PATH);

    // Trim off the base path and public file path.
    $public_file_path = PublicStream::basePath();
    $image_file_path = str_replace(base_path() . $public_file_path . '/', '', $path);

    // Account for images that are an image style derivative. In this case,
    // the URL has a prefix of the form: 'styles/STYLE_ID/SOURCE_SCHEME/'.
    // @todo Consider the corner case where the actual folder to save the
    // image has this in the path?
    $image_file_path = preg_replace('@styles/\w+/\w+/@', '', $image_file_path);
    $image_uri = 'public://' . $image_file_path;

    $file = $this->fileRepository->loadByUri($image_uri);

    if (!$file) {
      throw new \InvalidArgumentException("No image found for the path $image_url.");
    }

    return $file;
  }

}
