<?php

namespace Drupal\Tests\alt_text_import_csv\Kernel;

use Drupal\Core\Entity\EntityInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;
use Drupal\Tests\image\Kernel\ImageFieldCreationTrait;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Tests the alt text importer.
 *
 * @group alt_text_import_csv
 */
class AltTextImporterTest extends KernelTestBase {

  use EntityReferenceTestTrait;
  use ImageFieldCreationTrait;
  use MediaTypeCreationTrait;
  use NodeCreationTrait;
  use UserCreationTrait;

  /**
   * The modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'node',
    'file',
    'image',
    'media',
    'path',
    'path_alias',
    'alt_text_import_csv',
  ];

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('file');
    $this->installSchema('file', ['file_usage']);
    $this->installSchema('node', 'node_access');
    $this->installEntitySchema('node');
    $this->installEntitySchema('media');
    $this->installEntitySchema('path_alias');

    $this->entityTypeManager = $this->container->get('entity_type.manager');

    $this->setUpCurrentUser();
  }

  /**
   * Tests the alt text importer service with a media reference field.
   */
  public function testMediaField() {
    // Create the node type, media type, and reference field.
    $node_type_storage = $this->entityTypeManager->getStorage('node_type');
    $type = $node_type_storage->create([
      'type' => 'page',
      'name' => 'Page',
    ]);
    $type->save();

    $this->createMediaType('image', ['id' => 'test_image']);

    $this->createEntityReferenceField(
      'node',
      'page',
      'field_image',
      'Image',
      'media',
    );

    // Create the file, media, and node entities.
    $this->container->get('file_system')->copy($this->root . '/core/misc/druplicon.png', 'public://druplicon.png');
    $image_file = $this->entityTypeManager->getStorage('file')->create([
      'uri' => 'public://druplicon.png',
    ]);
    $image_file->save();

    $media = $this->entityTypeManager->getStorage('media')->create([
      'bundle' => 'test_image',
      'field_media_image' => $image_file,
    ]);
    $media->save();

    $node = $this->createNode([
      'type' => 'page',
      'field_image' => $media,
      'path' => ['alias' => '/foo'],
    ]);
    $node->save();

    /** @var \Drupal\path_alias\AliasRepositoryInterface $alias_repository */
    $alias_repository = $this->container->get('path_alias.repository');
    $stored_alias = $alias_repository->lookupBySystemPath('/' . $node->toUrl()->getInternalPath(), $node->language()->getId());
    $this->assertEquals('/foo', $stored_alias['alias']);

    /** @var \Drupal\alt_text_import_csv\AltTextImporter $alt_text_importer */
    $alt_text_importer = $this->container->get('alt_text_import_csv.alt_text_importer');
    // Get the image URL. Note that this looks totally broken because of a core
    // bug, but will work in AltTextImporter because of how other paths are
    // returned by the test system.
    // See https://www.drupal.org/project/drupal/issues/3401247.
    // This gets us a broken URL of the form
    // /vfs://root/sites/simpletest/38522002/files/druplicon.png.
    $image_url = $image_file->createFileUrl();
    // Add a token, as image URLs copied by users will have these.
    $image_url .= '?itok=5IYrswxV';

    $alt_text_importer->importAltText($image_url, 'http://example.com/foo', 'new alt text');

    $media = $this->reloadEntity($media);
    $this->assertEquals('new alt text', $media->field_media_image[0]->alt);
  }

  /**
   * Tests the alt text importer service with an image field.
   */
  public function testImageField() {
    // Create the node type, and reference field.
    $node_type_storage = $this->entityTypeManager->getStorage('node_type');
    $type = $node_type_storage->create([
      'type' => 'page',
      'name' => 'Page',
    ]);
    $type->save();

    $this->createImageField(
      'field_image',
      'page',
    );

    // Create the file, and node entities.
    $this->container->get('file_system')->copy($this->root . '/core/misc/druplicon.png', 'public://druplicon.png');
    $image_file = $this->entityTypeManager->getStorage('file')->create([
      'uri' => 'public://druplicon.png',
    ]);
    $image_file->save();

    $node = $this->createNode([
      'type' => 'page',
      'field_image' => $image_file,
      'path' => ['alias' => '/foo'],
    ]);
    $node->save();

    /** @var \Drupal\path_alias\AliasRepositoryInterface $alias_repository */
    $alias_repository = $this->container->get('path_alias.repository');
    $stored_alias = $alias_repository->lookupBySystemPath('/' . $node->toUrl()->getInternalPath(), $node->language()->getId());
    $this->assertEquals('/foo', $stored_alias['alias']);

    /** @var \Drupal\alt_text_import_csv\AltTextImporter $alt_text_importer */
    $alt_text_importer = $this->container->get('alt_text_import_csv.alt_text_importer');
    // Get the image URL. Note that this looks totally broken because of a core
    // bug, but will work in AltTextImporter because of how other paths are
    // returned by the test system.
    // See https://www.drupal.org/project/drupal/issues/3401247.
    // This gets us a broken URL of the form
    // /vfs://root/sites/simpletest/38522002/files/druplicon.png.
    $image_url = $image_file->createFileUrl();
    // Add a token, as image URLs copied by users will have these.
    $image_url .= '?itok=5IYrswxV';

    $alt_text_importer->importAltText($image_url, 'http://example.com/foo', 'new alt text');

    $node = $this->reloadEntity($node);
    $this->assertEquals('new alt text', $node->field_image[0]->alt);
  }

  /**
   * @todo Remove when https://www.drupal.org/project/drupal/issues/3224276
   * finally gets in.
   */
  protected function reloadEntity(EntityInterface $entity) {
    $controller = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    $controller->resetCache([$entity->id()]);
    return $controller->load($entity->id());
  }

}
